﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using com.google.zxing;
using com.google.zxing.common;
using Plugins;

namespace TagTracker
{
    public partial class Configure : Form
    {
        private readonly MultiFormatWriter _writer = new MultiFormatWriter();
        public Main MainClass;

        public Configure()
        {
            InitializeComponent();
        }

        public enum RGBcoefficientType
        {
            BT709,
        }
        public class GrayScaleCoef
        {
            public double cr;
            public double cg;
            public double cb;

            public GrayScaleCoef(double cr, double cg, double cb)
            {
                this.cr = cr;
                this.cg = cg;
                this.cb = cb;
            }
        }
        private static unsafe void GetGrayScaleImage(BitmapData sourceData, BitmapData destinationData, GrayScaleCoef coef)
        {
            // get width and height
            int width = sourceData.Width;
            int height = sourceData.Height;

            PixelFormat srcPixelFormat = sourceData.PixelFormat;

            if (
                (srcPixelFormat == PixelFormat.Format24bppRgb) ||
                (srcPixelFormat == PixelFormat.Format32bppRgb) ||
                (srcPixelFormat == PixelFormat.Format32bppArgb))
            {
                int pixelSize = (srcPixelFormat == PixelFormat.Format24bppRgb) ? 3 : 4;
                int srcOffset = sourceData.Stride - width * pixelSize;
                int dstOffset = destinationData.Stride - width;

                // do the job
                byte* src = (byte*)sourceData.Scan0.ToPointer();
                byte* dst = (byte*)destinationData.Scan0.ToPointer();

                // for each line
                for (int y = 0; y < height; y++)
                {
                    // for each pixel
                    for (int x = 0; x < width; x++, src += pixelSize, dst++)
                    {
                        *dst = (byte)(coef.cr * src[2] + coef.cg * src[1] + coef.cb * src[0]);
                    }
                    src += srcOffset;
                    dst += dstOffset;
                }
            }
        }
        public static Bitmap GraySacle(Bitmap image, RGBcoefficientType otype)
        {
            Bitmap dstimage = null;
            try
            {
                GrayScaleCoef coef = otype == RGBcoefficientType.BT709 ? new GrayScaleCoef(0.2125, 0.7154, 0.0721) : new GrayScaleCoef(0.5, 0.419, 0.081);
                BitmapData sourceData = image.LockBits(new Rectangle(0, 0, image.Width, image.Height), ImageLockMode.ReadOnly, image.PixelFormat);
                dstimage = CreateGrayscaleImage(image.Width, image.Height);
                BitmapData destinationData = dstimage.LockBits(new Rectangle(0, 0, dstimage.Width, dstimage.Height), ImageLockMode.ReadWrite, dstimage.PixelFormat);
                GetGrayScaleImage(sourceData, destinationData, coef);
                dstimage.UnlockBits(destinationData);
                image.UnlockBits(sourceData);
            }
            catch
            {

            }
            return dstimage;
        }
        public static Bitmap CreateGrayscaleImage(int width, int height)
        {
            // create new image
            var image = new Bitmap(width, height, PixelFormat.Format8bppIndexed);
            // set palette to grayscale
            // get palette
            var cp = image.Palette;
            // init palette
            for (int i = 0; i < 256; i++)
            {
                cp.Entries[i] = Color.FromArgb(i, i, i);
            }
            // set palette back
            image.Palette = cp;
            // return new image
            return image;
        }

        
        #region ENCODE SECTION
        private static unsafe Bitmap ConvertByteMarixToImage(ByteMatrix bm)
        {
            Bitmap image = CreateGrayscaleImage(bm.Width, bm.Height);

            BitmapData sourceData = image.LockBits(new Rectangle(0, 0, image.Width, image.Height), ImageLockMode.ReadOnly, image.PixelFormat);
            int width = sourceData.Width;
            int height = sourceData.Height;
            int srcOffset = sourceData.Stride - width;
            var src = (byte*)sourceData.Scan0.ToPointer();
            for (int y = 0; y < height; y++)
            {
                // for each pixel
                for (int x = 0; x < width; x++, src++)
                {
                    *src = (byte)bm.Array[y][x];
                }
                src += srcOffset;
            }
            image.UnlockBits(sourceData);
            return image;
        }
        #endregion

        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                BarcodeFormat fm;
                switch (ddlGenType.SelectedIndex)
                {
                    case 0: fm = BarcodeFormat.EAN_8;
                        break;
                    case 1: fm = BarcodeFormat.EAN_13;
                        break;
                    case 2: fm = BarcodeFormat.QR_CODE;
                        break;
                    default:
                        fm = BarcodeFormat.EAN_8;
                        break;
                }
                var bt = _writer.encode(txtGenerate.Text, fm, (int)numWidth.Value, (int)numHeight.Value);

                var bm = ConvertByteMarixToImage(bt);
                pbResult.Image = bm;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void Configure_Load(object sender, EventArgs e)
        {
            chkUPCE.Checked = MainClass.FormatUpcE;
            chkUPCA.Checked = MainClass.FormatUpcA;
            chkQRCode.Checked = MainClass.FormatQrCode;
            chkPDF417.Checked = MainClass.FormatPdf417;
            chkEAN8.Checked = MainClass.FormatEan8;
            chkEAN13.Checked = MainClass.FormatEan13;
            chkDataMatrix.Checked = MainClass.FormatDatamatrix;
            chkCodeITF.Checked = MainClass.FormatItf;
            chkCode39.Checked = MainClass.FormatCode39;
            chkCode128.Checked = MainClass.FormatCode128;

            chkTryHarder.Checked = MainClass.Intensive;

            ddlGenType.SelectedIndex = 2;
            rtbAlertText.Text = MainClass.AlertText;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            MainClass.FormatUpcE = chkUPCE.Checked;
            MainClass.FormatUpcA = chkUPCA.Checked;
            MainClass.Intensive = chkTryHarder.Checked;
            MainClass.FormatQrCode = chkQRCode.Checked;
            MainClass.FormatPdf417 = chkPDF417.Checked;
            MainClass.FormatEan8 = chkEAN8.Checked;
            MainClass.FormatEan13 = chkEAN13.Checked;
            MainClass.FormatDatamatrix = chkDataMatrix.Checked;
            MainClass.FormatItf = chkCodeITF.Checked;
            MainClass.FormatCode39 = chkCode39.Checked;
            MainClass.FormatCode128 = chkCode128.Checked;
            MainClass.AlertText = rtbAlertText.Text.Replace("|", "");
            DialogResult = DialogResult.OK;
            Close();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            if (pbResult.Image != null)
            {
                var sdlg = new SaveFileDialog
                {
                    Filter = "BMP files (*.bmp)|*.bmp|All files (*.*)|*.*"
                };
                if (sdlg.ShowDialog() == DialogResult.OK)
                {
                    pbResult.Image.Save(sdlg.FileName);
                }
            }
        }
    }
}
