﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Threading;
using System.Windows.Forms;
using com.google.zxing;
using com.google.zxing.common;
using TagTracker;

namespace Plugins
{
    public class Main : IDisposable
    {
        private bool _disposed;
        private string _config = "";
        public string Alert;

        private readonly Font _textFont = new Font(FontFamily.GenericSansSerif, 11, FontStyle.Regular, GraphicsUnit.Pixel);
        private readonly Brush _backBrush = new SolidBrush(Color.FromArgb(128, 255, 255, 255));
        private readonly Brush _textBrush = new SolidBrush(Color.Black);
        private List<String> _alertList;

        internal bool FormatDatamatrix;
        internal bool FormatQrCode = true;
        internal bool FormatPdf417;
        internal bool FormatUpcE;
        internal bool FormatUpcA;
        internal bool FormatCode128;
        internal bool FormatCode39;
        internal bool FormatItf;
        internal bool FormatEan8;
        internal bool FormatEan13;

        internal bool Intensive;
        internal string AlertText;
        
        private Thread _processor;
        private string _message = "";
        private DateTime _lastmessage = DateTime.MinValue;
        internal static Brush OverlayBrush = new SolidBrush(Color.FromArgb(120, 0, 0, 0));
        internal static Font Drawfont = new Font(FontFamily.GenericSansSerif, 12, FontStyle.Regular, GraphicsUnit.Pixel);
        internal static Brush DrawBrush = new SolidBrush(Color.White);
        public Bitmap currentFrame;

        internal string Message
        {
            get
            {
                return _message;
            }
            set
            {
                _message = value;
                _lastmessage = DateTime.Now;
            }
        }

        /// <summary>
        /// Arguments for new frame event from Audio source.
        /// </summary>
        /// 
        public class NewFrameEventArgs : EventArgs
        {
            private readonly Bitmap _frame;
            public NewFrameEventArgs(Bitmap frame)
            {
                _frame = frame;
            }
            public Bitmap Frame
            {
                get { return _frame; }
            }
        }

        public Bitmap ProcessFrame(Bitmap frame)
        {
            if (_processor == null || !_processor.IsAlive)
            {
                try
                {
                    if (currentFrame != null)
                    {
                        currentFrame.Dispose();
                        currentFrame = null;
                    }

                    currentFrame = new Bitmap(frame);
                    _processor = new Thread(RunScanner) { Name = "Barcode Detector", IsBackground = true };
                    _processor.Start();
                }
                catch (Exception ex)
                {
                    string m = ex.Message;

                }

            }

            if (Message != "")
            {
                var sec = (DateTime.Now - _lastmessage).TotalMilliseconds;
                if (sec < 5000)
                {
                    double y = 20;
                    if (sec > 3500)
                        y = 20 - ((sec - 3500d) / 1500d) * 20;
                    var g = Graphics.FromImage(frame);
                    g.FillRectangle(OverlayBrush, 0, frame.Height - Convert.ToInt32(y), frame.Width, 20);
                    g.DrawString(Message, Drawfont, DrawBrush, 5, frame.Height - Convert.ToInt32(y) + 2);
                    g.Dispose();
                }
                else
                    Message = "";
            }


            return frame;
        }

        private void RunScanner()
        {
            var mBarcodeReader = new MultiFormatReader();

            var bmp = currentFrame;

            var hints = new Hashtable();
            var fmts = new ArrayList();
            if (FormatDatamatrix)
                fmts.Add(BarcodeFormat.DATAMATRIX);
            if (FormatQrCode)
                fmts.Add(BarcodeFormat.QR_CODE);
            if (FormatPdf417)
                fmts.Add(BarcodeFormat.PDF417);
            if (FormatUpcE)
                fmts.Add(BarcodeFormat.UPC_E);
            if (FormatUpcA)
                fmts.Add(BarcodeFormat.UPC_A);
            if (FormatCode128)
                fmts.Add(BarcodeFormat.CODE_128);
            if (FormatCode39)
                fmts.Add(BarcodeFormat.CODE_39);
            if (FormatItf)
                fmts.Add(BarcodeFormat.ITF);
            if (FormatEan8)
                fmts.Add(BarcodeFormat.EAN_8);
            if (FormatEan13)
                fmts.Add(BarcodeFormat.EAN_13);

            if (Intensive)
                hints.Add(DecodeHintType.TRY_HARDER, true);

            hints.Add(DecodeHintType.POSSIBLE_FORMATS, fmts);

            Result rawResult = null;
            var r = new RGBLuminanceSource(bmp, bmp.Width, bmp.Height);
            var x = new GlobalHistogramBinarizer(r);
            var bitmap = new BinaryBitmap(x);

            try
            {
                rawResult = mBarcodeReader.decode(bitmap, hints);
            }
            catch (ReaderException e)
            {

            }
            catch (Exception ex)
            {
                string m = ex.Message;
            }

            if (rawResult != null)
            {
                if (!String.IsNullOrEmpty(rawResult.Text))
                {
                    string m = rawResult.Text + " (" + rawResult.BarcodeFormat.Name + ")";
                    Message = m;

                    if (_alertList.Contains(rawResult.Text))
                    {
                        Alert = m;
                    }
                    else
                    {
                        Alert = "";
                    }
                    
                }
                else
                {
                    Alert = "";
                }
            }
            else
            {
                Alert = "";
            }
            
            bmp.Dispose();
            bmp = null;
        }

        public string Configuration
        {
            get { return _config; }
            set
            {
                _config = value;
                InitConfig();
            }
        }

        private void InitConfig()
        {
            if (_config != "")
            {
                string[] cfg = _config.Split('|');

                FormatDatamatrix = Convert.ToBoolean(cfg[0]);
                FormatQrCode = Convert.ToBoolean(cfg[1]);
                FormatPdf417 = Convert.ToBoolean(cfg[2]);
                FormatUpcE = Convert.ToBoolean(cfg[3]);
                FormatUpcA = Convert.ToBoolean(cfg[4]);
                FormatCode128 = Convert.ToBoolean(cfg[5]);
                FormatCode39 = Convert.ToBoolean(cfg[6]);
                FormatItf = Convert.ToBoolean(cfg[7]);
                FormatEan8 = Convert.ToBoolean(cfg[8]);
                FormatEan13 = Convert.ToBoolean(cfg[9]);

                Intensive = Convert.ToBoolean(cfg[10]);
                AlertText = cfg[11];
                _alertList = AlertText.Split(Environment.NewLine.ToCharArray()).ToList();
            }
        }

        public string Configure()
        {
            var cfg = new Configure
            {
                MainClass = this
            };

            if (cfg.ShowDialog() == DialogResult.OK)
            {
                _config = FormatDatamatrix + "|" + FormatQrCode + "|" + FormatPdf417 + "|" + FormatUpcE + "|" +
                          FormatUpcA + "|" + FormatCode128 + "|" + FormatCode39 + "|" + FormatItf + "|" + FormatEan8 +
                          "|" + FormatEan13 + "|" + Intensive + "|" + AlertText;
                InitConfig();
            }
            return Configuration;
        }

        //Implement IDisposable.
        public void Dispose()
        {
            Dispose(true);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                if (disposing)
                {
                    // Free other state (managed objects).
                    _textFont.Dispose();
                    _backBrush.Dispose();
                    _textBrush.Dispose();
                }

                // Free your own state (unmanaged objects).
                // Set large fields to null.
                _disposed = true;
            }
        }
    }
}
